use std::{
    env,
    fs,
    io::{self, ErrorKind},
    path::{Path, PathBuf},
};

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error("No .env file found at {0:?}")]
    NoEnvFile(PathBuf),

    #[error("IO error when opening {0:?}: {1}")]
    IoError(PathBuf, io::Error),
}

pub fn set_env_from_file(root: Option<&str>) -> Result<(), Error> {
    let root = Path::new(root.unwrap_or("."));
    let path = root.join(".env");

    match fs::read_to_string(&path) {
        Ok(contents) => {
            for line in contents.lines() {
                let mut parts = line.split('=');

                let key = parts.next();
                let value = parts.next();

                if let (Some(key), Some(value)) = (key, value) {
                    // TODO:
                    // Not sure if this should override command line env variables or other way
                    // around.
                    env::set_var(key, value);
                }
            }

            Ok(())
        },

        Err(error) => match error.kind() {
            ErrorKind::NotFound => Err(Error::NoEnvFile(path)),

            _ => Err(Error::IoError(path, error)),
        },
    }
}
